/*

DS Delay Line v0.3
(c) 2009  Avelino Herrera Morales

ARM7 code

*/


#include <stdlib.h>
#include <nds.h>


#define  SAMPLE_RATE          22050
#define  BUFFER_SIZE_SECONDS  8
#define  BUFFER_SIZE_SAMPLES  (SAMPLE_RATE * BUFFER_SIZE_SECONDS)
#define  BUFFER_SIZE_BYTES    (BUFFER_SIZE_SAMPLES << 1)
#define  BUFFER_SIZE_WORDS    (BUFFER_SIZE_SAMPLES >> 1)
#define  SAMPLES_PER_MS       (SAMPLE_RATE / 1000)
#define  MIC_GAIN             4


u32 *soundBuffer;
u32 sampleCounter;
u32 delaySamples;
u32 soundTimingRate;
s32 micGain;
u32 startDelay;
u32 delayRunning;
u32 insertPointBackup;


void userDataReceived01(u32 value, void *userData) {
	// the fifo receives a value measured in miliseconds
	delaySamples = value * SAMPLES_PER_MS;
}


void userDataReceived02(u32 value, void *userData) {
	// the fifo receives a value for the mic gain (24.8 fixed point format)
	micGain = (s32) value;
}


void VcountHandler() {
	inputGetAndSend();
}


void timer0Handler() {
	/*
	s32 micData;
	u32 insertPoint;
	s16 *p;

	if (sampleCounter >= BUFFER_SIZE_SAMPLES)
		sampleCounter = 0;
	micData = (s32) micReadData12();
	micData = (micData - 2048) << 4;
	insertPoint = (sampleCounter + delaySamples) % BUFFER_SIZE_SAMPLES;
	p = (s16 *) soundBuffer;
	micData = (micData * micGain) / 256;
	if (micData > 32767)
		micData = 32767;
	else if (micData < -32768)
		micData = -32768;
	p[insertPoint] = micData;
	*/
	sampleCounter++;
}


void insertMicSamples(u32 *insertPointBackup, u32 counter) {
	s32 micData;
	u32 insertPoint;
	u32 aux;
	s16 *p;
	u32 i;

	micData = (s32) micReadData12();
	micData = (micData - 2048) << 4;
	micData = (micData * micGain) / 256;
	if (micData > 32767)
		micData = 32767;
	else if (micData < -32768)
		micData = -32768;
	p = (s16 *) soundBuffer;
	aux = (*insertPointBackup + delaySamples) % BUFFER_SIZE_SAMPLES;
	for (i = 0; i < counter; i++) {
		p[aux] = micData;
		aux = (aux + 1) % BUFFER_SIZE_SAMPLES;
	}
	*insertPointBackup += counter;
}


int main() {
	int i;
	s16 *p;

	powerOn(POWER_SOUND);
	REG_SOUNDCNT = SOUND_VOL(127) | SOUND_ENABLE;

	irqInit();
	fifoInit();
	// read User Settings from firmware
	readUserSettings();
	// start the RTC tracking IRQ
	initClockIRQ();
	// set the IRQ handlers
	SetYtrigger(80);
	irqSet(IRQ_VCOUNT, VcountHandler);
	irqSet(IRQ_TIMER0, timer0Handler);
	irqEnable(IRQ_VCOUNT | IRQ_TIMER0);
	// set fifo receive callback
	fifoSetValue32Handler(FIFO_USER_01, userDataReceived01, NULL);
	fifoSetValue32Handler(FIFO_USER_02, userDataReceived02, NULL);
	// init buffer and some other variables
	// soundBuffer = (u32 *) malloc(BUFFER_SIZE_BYTES);
	soundBuffer = (u32 *) (MAINRAM8 + 131072);
	soundTimingRate = SOUND_FREQ(SAMPLE_RATE);
	micOn();
	sampleCounter = 0;
	insertPointBackup = 0;
	delaySamples = 0;
	micGain = 256;  // 1 in 24.8 fixed point format
	startDelay = 1;
	delayRunning = 0;
	while (1) {
		if (!delayRunning) {
			if (startDelay) {
				// start sound playing
				p = (s16 *) soundBuffer;
				for (i = 0; i < BUFFER_SIZE_SAMPLES; i++, p++)
					*p = 0;
				SCHANNEL_TIMER(0) = soundTimingRate;
				SCHANNEL_SOURCE(0) = (u32) soundBuffer;
				SCHANNEL_REPEAT_POINT(0) = 0;
				SCHANNEL_LENGTH(0) = BUFFER_SIZE_BYTES >> 2;
				SCHANNEL_CR(0) = SCHANNEL_ENABLE | SOUND_REPEAT | SOUND_FORMAT_16BIT | SOUND_VOL(127) | SOUND_PAN(64);
				// start mic capture timer
				TIMER_DATA(0) = soundTimingRate * 2;
				TIMER_CR(0) = TIMER_DIV_1 | TIMER_ENABLE | TIMER_IRQ_REQ;
				delayRunning = 1;
			}
		}
		else {
			if (!startDelay) {
				SCHANNEL_CR(0) = 0;
				TIMER_CR(0) = 0;
				delayRunning = 0;
			}
			else {
				u32 currentSampleCounter = sampleCounter;
				if (currentSampleCounter > 0) {
					insertMicSamples(&insertPointBackup, currentSampleCounter);
					sampleCounter -= currentSampleCounter;
				}
			}
		}
		//swiWaitForVBlank();
	}
}


