/*

DS Delay Line v0.3
(c) 2009  Avelino Herrera Morales

ARM9 code

*/


#include <nds.h>
#include <stdio.h>
#include "font.h"


#define  DELAY_MS_MAX  7800
#define  GAIN_MIN      (1 * 256)
#define  GAIN_MAX      (5 * 256)


u32 fifoValue;
u32 delayMs;
u32 micGain;
u16 *mainScreenMap;
u16 *subScreenMap;
u16 *delay_fader_sprite;
u16 *gain_fader_sprite;
char aux_str[10];
s32 gain_int_part;
s32 gain_frac_part;


void mainScreenInit() {
	int bg;
	u16 *tiles;
	u16 i;

	videoSetMode(MODE_0_2D | DISPLAY_BG0_ACTIVE);
	vramSetBankA(VRAM_A_MAIN_BG);
	bg = bgInit(0, BgType_Text8bpp, BgSize_T_256x256, 0, 1);
	// init the tiles
	tiles = bgGetGfxPtr(bg);
	dmaCopy(font_tiles, tiles, (128 * 8 * 8));
	// init map
	mainScreenMap = bgGetMapPtr(bg);
	dmaFillWords(0, mainScreenMap, (32 * 24 * 2));
	// palette
	for (i = 0; i < 256; i++) {
		BG_PALETTE[i] = font_palette[i];
		SPRITE_PALETTE[i] = font_palette[i];
	}
}


void subScreenInit() {
	int bg;
	u16 *tiles;
	u16 i;

	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);
	vramSetBankC(VRAM_C_SUB_BG);
	bg = bgInitSub(0, BgType_Text8bpp, BgSize_T_256x256, 0, 1);
	// init the tiles
	tiles = bgGetGfxPtr(bg);
	dmaCopy(font_tiles, tiles, (128 * 8 * 8));
	// init map
	subScreenMap = bgGetMapPtr(bg);
	dmaFillWords(0, subScreenMap, (32 * 24 * 2));
	// palette
	for (i = 0; i < 256; i++) {
		BG_PALETTE_SUB[i] = font_palette[i];
		SPRITE_PALETTE_SUB[i] = font_palette[i];
	}
	vramSetBankD(VRAM_D_SUB_SPRITE);
	oamInit(&oamSub, SpriteMapping_Bmp_1D_128, false);
	oamEnable(&oamSub);
}


void screenPrint(s32 x, s32 y, char *s, u16 *map) {
	s32 offset = (y * 32) + x;
	while (*s != 0) {
		map[offset] = (u16) *s;
		offset++;
		s++;
	}
}


void mprint(s32 x, s32 y, char *s) {
	screenPrint(x, y, s, mainScreenMap);
}


void sprint(s32 x, s32 y, char *s) {
	screenPrint(x, y, s, subScreenMap);
}


void delay_fader_init() {
	delay_fader_sprite = oamAllocateGfx(&oamSub, SpriteSize_8x32, SpriteColorFormat_256Color);
	dmaFillWords(0xFFFFFFFF, delay_fader_sprite, 224);
}


void delay_fader_locate(s32 x) {
	oamSet(&oamSub, 0, 10 + x, 50, 0, 10, SpriteSize_8x32, SpriteColorFormat_256Color, delay_fader_sprite, -1, false, false, false, false, false);
}


s32 delay_fader_update(s32 tx, s32 ty) {
	if ((tx < 10) || (tx > 246) || (ty < 50) || (ty > 77))
		return -1;
	tx -= 14;
	if (tx < 0)
		tx = 0;
	else if (tx > 228)
		tx = 228;
	delay_fader_locate(tx);
	return (tx * DELAY_MS_MAX) / 228;
}


void gain_fader_init() {
	gain_fader_sprite = oamAllocateGfx(&oamSub, SpriteSize_8x32, SpriteColorFormat_256Color);
	dmaFillWords(0xFFFFFFFF, gain_fader_sprite, 224);
}


void gain_fader_locate(s32 x) {
	oamSet(&oamSub, 1, 26 + x, 130, 0, 10, SpriteSize_8x32, SpriteColorFormat_256Color, gain_fader_sprite, -1, false, false, false, false, false);
}


s32 gain_fader_update(s32 tx, s32 ty) {
	if ((tx < 26) || (tx > 230) || (ty < 130) || (ty > 157))
		return -1;
	tx -= 30;
	if (tx < 0)
		tx = 0;
	else if (tx > 196)
		tx = 196;
	gain_fader_locate(tx);
	return GAIN_MIN + ((tx * (GAIN_MAX - GAIN_MIN)) / 196);
}


int main(void) {
	touchPosition touch;

	// irqInit();
	// irqEnable(IRQ_VBLANK);

	REG_EXMEMCNT |= ARM7_MAIN_RAM_PRIORITY;

	mainScreenInit();
	subScreenInit();

	mprint(7, 3, "DS Delay Line v0.4");
	mprint(1, 10, "@ 2010 Avelino Herrera Morales");
	mprint(4, 16, "http://nds.atlantes.org");

	sprint(6, 4, "Delay amount:");
	sprint(1, 6,  "\x01\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x03");
	sprint(1, 7,  "\x04                            \x05");
	sprint(1, 8, "\x04                            \x05");
	sprint(1, 9, "\x06\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x08");

	sprint(6, 14, "Gain amount: x");
	sprint(3, 16,  "\x01\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x03");
	sprint(3, 17,  "\x04                        \x05");
	sprint(3, 18, "\x04                        \x05");
	sprint(3, 19, "\x06\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x07\x08");

	delay_fader_init();
	delay_fader_locate(0);
	gain_fader_init();
	gain_fader_locate(0);

	delayMs = 0;
	sprintf(aux_str, "%4d ms", (int) delayMs);
	sprint(20, 4, aux_str);
	micGain = GAIN_MIN;
	gain_int_part = micGain >> 8;
	gain_frac_part = ((micGain & 0xFF) * 1000) / 256;
	sprintf(aux_str, "%1d.%03d", (int) gain_int_part, (int) gain_frac_part);
	sprint(21, 14, aux_str);
	while(1) {
		u16 touch_x, touch_y; 
		s32 aux;
		touchRead(&touch);
		touch_x = touch.px >> 3;
		touch_y = touch.py >> 3;
		aux = delay_fader_update(touch.px, touch.py);
		if (aux >= 0) {
			delayMs = aux;
			fifoSendValue32(FIFO_USER_01, (u32) delayMs);
			sprintf(aux_str, "%4d ms", (int) delayMs);
			sprint(20, 4, aux_str);
		}
		aux = gain_fader_update(touch.px, touch.py);
		if (aux >= 0) {
			micGain = aux;
			fifoSendValue32(FIFO_USER_02, (u32) micGain);
			gain_int_part = micGain >> 8;
			gain_frac_part = ((micGain & 0xFF) * 1000) / 256;
			sprintf(aux_str, "%1d.%03d", (int) gain_int_part, (int) gain_frac_part);
			sprint(21, 14, aux_str);
		}
		swiWaitForVBlank();
		oamUpdate(&oamSub);
	}

	return 0;
}
